# -*- coding: UTF-8 -*-

import re
import datetime

_nanosecond_size  = 1
_microsecond_size = 1000 * _nanosecond_size
_millisecond_size = 1000 * _microsecond_size
_second_size      = 1000 * _millisecond_size
_minute_size      = 60   * _second_size
_hour_size        = 60   * _minute_size
_day_size         = 24   * _hour_size
_week_size        = 7    * _day_size
_month_size       = 30   * _day_size
_year_size        = 365  * _day_size

units = {
    "ns": _nanosecond_size,
    "us": _microsecond_size,
    "µs": _microsecond_size,
    "μs": _microsecond_size,
    "ms": _millisecond_size,
    "s":  _second_size,
    "m":  _minute_size,
    "h":  _hour_size,
    "d":  _day_size,
    "w":  _week_size,
    "mm": _month_size,
    "y":  _year_size,
}

_duration_re = re.compile(r'([\d\.]+)([a-zµμ]+)')


class DurationError(ValueError):
    """duration error"""


def from_str(duration):
    """Parse a duration string to a datetime.timedelta"""

    original = duration

    if duration in ("0", "+0", "-0"):
        return datetime.timedelta()

    sign = 1
    if duration and duration[0] in '+-':
        if duration[0] == '-':
            sign = -1
        duration = duration[1:]

    matches = list(_duration_re.finditer(duration))
    if not matches:
        raise DurationError("Invalid duration {}".format(original))
    if matches[0].start() != 0 or matches[-1].end() != len(duration):
        raise DurationError(
            'Extra chars at start or end of duration {}'.format(original))

    total = 0
    for match in matches:
        value, unit = match.groups()
        if unit not in units:
            raise DurationError(
                "Unknown unit {} in duration {}".format(unit, original))
        try:
            total += float(value) * units[unit]
        except Exception:
            raise DurationError(
                "Invalid value {} in duration {}".format(value, original))

    microseconds = total / _microsecond_size
    return datetime.timedelta(microseconds=sign * microseconds)

def to_str(delta, extended=False):
    """Format a datetime.timedelta to a duration string"""

    total_seconds = delta.total_seconds()
    sign = "-" if total_seconds < 0 else ""
    nanoseconds = round(abs(total_seconds * _second_size), 0)

    if abs(total_seconds) < 1:
        result_str = _to_str_small(nanoseconds, extended)
    else:
        result_str = _to_str_large(nanoseconds, extended)

    return "{}{}".format(sign, result_str)


def _to_str_small(nanoseconds, extended):

    result_str = ""

    if not nanoseconds:
        return "0"

    milliseconds = int(nanoseconds / _millisecond_size)
    if milliseconds:
        nanoseconds -= _millisecond_size * milliseconds
        result_str += "{:g}ms".format(milliseconds)

    microseconds = int(nanoseconds / _microsecond_size)
    if microseconds:
        nanoseconds -= _microsecond_size * microseconds
        result_str += "{:g}us".format(microseconds)

    if nanoseconds:
        result_str += "{:g}ns".format(nanoseconds)

    return result_str


def _to_str_large(nanoseconds, extended):

    result_str = ""

    if extended:

        years = int(nanoseconds / _year_size)
        if years:
            nanoseconds -= _year_size * years
            result_str += "{:g}y".format(years)

        months = int(nanoseconds / _month_size)
        if months:
            nanoseconds -= _month_size * months
            result_str += "{:g}mm".format(months)

        days = int(nanoseconds / _day_size)
        if days:
            nanoseconds -= _day_size * days
            result_str += "{:g}d".format(days)

    hours = int(nanoseconds / _hour_size)
    if hours:
        nanoseconds -= _hour_size * hours
        result_str += "{:g}h".format(hours)

    minutes = int(nanoseconds / _minute_size)
    if minutes:
        nanoseconds -= _minute_size * minutes
        result_str += "{:g}m".format(minutes)

    seconds = float(nanoseconds) / float(_second_size)
    if seconds:
        nanoseconds -= _second_size * seconds
        result_str += "{:g}s".format(seconds)

    return result_str



from setuptools import setup

setup(
    name         = 'durationpy',
    description  = 'Module for converting between datetime.timedelta and Go\'s Duration strings.',
    url          = 'https://github.com/icholy/durationpy',
    author       = 'Ilia Choly',
    author_email = 'ilia.choly@gmail.com',
    download_url = 'https://github.com/icholy/durationpy/tarball/0.9',
    version      = '0.9',
    packages     = ['durationpy'],
    license      = 'MIT'
)




import unittest
import durationpy
from datetime import timedelta

millisecond = 1
second      = 1000 * millisecond
minute      = 60   * second
hour        = 60   * minute
day         = 24   * hour
week        = 7    * day
month       = 30   * day
year        = 365  * day

cases = [

  # simple
  ["0", True, 0],
  ["5s", True, 5 * second],
  ["30s", True, 30 * second],
  ["1478s", True, 1478 * second],
  
  # sign
  ["-5s", True, -5 * second],
  ["+5s", True, 5 * second],
  ["-0", True, 0],
  ["+0", True, 0],
  
  # decimal
  ["5.0s", True, 5 * second],
  ["5.6s", True, 5*second + 600*millisecond],
  ["5.s", True, 5 * second],
  [".5s", True, 500 * millisecond],
  ["1.0s", True, 1 * second],
  ["1.00s", True, 1 * second],
  ["1.004s", True, 1*second + 4*millisecond],
  ["1.0040s", True, 1*second + 4*millisecond],
  ["100.00100s", True, 100*second + 1*millisecond],
  
  # different units
  ["13ms", True, 13 * millisecond],
  ["14s", True, 14 * second],
  ["15m", True, 15 * minute],
  ["16h", True, 16 * hour],
  ["11d", True, 11 * day],
  ["10w", True, 10 * week],
  
  # composite durations
  ["3h30m", True, 3*hour + 30*minute],
  ["10.5s4m", True, 4*minute + 10*second + 500*millisecond],
  ["-2m3.4s", True, -(2*minute + 3*second + 400*millisecond)],
  ["1h2m3s4ms", True, 1*hour + 2*minute + 3*second + 4*millisecond],
  ["10w5d39h9m14.425s", True, 10*week + 5*day + 39*hour + 9*minute + 14*second + 425*millisecond],
  
  # large value
  ["52763797000ms", True, 52763797000 * millisecond],

  # small value
  # Use multiplication to preserve rounding errors
  ["492us", True, timedelta(microseconds=492).total_seconds() * 1000],
  
  # errors
  ["", False, 0],
  ["3", False, 0],
  ["-", False, 0],
  ["s", False, 0],
  [".", False, 0],
  ["-.", False, 0],
  [".s", False, 0],
  ["+.s", False, 0],
  ["X3h", False, 0],
  ["3hY", False, 0],
  ["X72h3m0.5msY", False, 0],
  ["+X3h", False, 0],
  ["-X3h", False, 0],

  # extended
  ["5y2mm", True, 5*year + 2*month],
  ["7d", True, 7*day],
  ["1y4w1h", True, 1*year + 4*week + 1*hour],
  ["-7d", True, -7*day]
]

class DurationTest(unittest.TestCase):

    def test_parser(self):
        for [input, passes, expected] in cases:
            if passes:
                actual = durationpy.from_str(input).total_seconds() * 1000
                self.assertEqual(
                    expected, actual,
                    "{}, expecting {}, got {}".format(input, expected, actual)) 
            else:
                with self.assertRaises(durationpy.DurationError, msg=repr(input)) as context:
                    durationpy.from_str(input)
                self.assertIn(input, str(context.exception), msg="error did not contain input")

    def test_formatter(self):
        for [input, passes, expected] in cases:
            if passes:
                dt = durationpy.from_str(input)
                ds = durationpy.to_str(dt)
                actual = durationpy.from_str(ds).total_seconds() * 1000
                self.assertEqual(
                    expected, actual,
                    "{}, expecting {}, got {}".format(input, expected, actual)) 

if __name__ == '__main__':
    unittest.main()
